@@ -31,12 +31,16 @@ defined('MOODLE_INTERNAL') || die();
 require_once($CFG->dirroot.'/cache/locallib.php');
 
 /**
- * Override the default cache configuration for our own maniacle purposes.
+ * Override the default cache configuration for our own maniacal purposes.
  *
- * @copyright  2012 Sam Hemelryk
- * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
+ * This class was originally named cache_config_phpunittest but was renamed in 2.9
+ * because it is used for both unit tests and acceptance tests.
+ *
+ * @since 2.9
+ * @copyright 2012 Sam Hemelryk
+ * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-class cache_config_phpunittest extends cache_config_writer {
+class cache_config_testing extends cache_config_writer {
 
     /**
      * Creates the default configuration and saves it.
@@ -61,14 +65,13 @@ class cache_config_phpunittest extends cache_config_writer {
         $appdefine = defined('TEST_CACHE_USING_APPLICATION_STORE') ? TEST_CACHE_USING_APPLICATION_STORE : false;
         if ($appdefine !== false && preg_match('/^[a-zA-Z][a-zA-Z0-9_]+$/', $appdefine)) {
             $expectedstore = $appdefine;
-            $expecteddefine = 'TEST_CACHESTORE_'.strtoupper($expectedstore).'_TESTSERVERS';
             $file = $CFG->dirroot.'/cache/stores/'.$appdefine.'/lib.php';
             $class = 'cachestore_'.$appdefine;
             if (file_exists($file)) {
                 require_once($file);
             }
-            if (defined($expecteddefine) && class_exists($class)) {
-                /** @var cache_store $class */
+            if (class_exists($class) && $class::ready_to_be_used_for_testing()) {
+                /* @var cache_store $class */
                 $writer->configstores['test_application'] = array(
                     'use_test_store' => true,
                     'name' => 'test_application',
@@ -136,10 +139,10 @@ class cache_config_phpunittest extends cache_config_writer {
 
         if (!empty($CFG->altcacheconfigpath)) {
 
-            if  (defined('PHPUNIT_TEST') && PHPUNIT_TEST &&
-                (!defined('TEST_CACHE_USING_ALT_CACHE_CONFIG_PATH') || !TEST_CACHE_USING_ALT_CACHE_CONFIG_PATH)) {
-                // We're within a unit test, but TEST_CACHE_USING_ALT_CACHE_CONFIG_PATH has not being defined or is
-                // false, we want to use the default.
+            // No need to check we are within a test here, this is the cache config class that gets used
+            // only when one of those is true.
+            if  (!defined('TEST_CACHE_USING_ALT_CACHE_CONFIG_PATH') || !TEST_CACHE_USING_ALT_CACHE_CONFIG_PATH) {
+                // TEST_CACHE_USING_ALT_CACHE_CONFIG_PATH has not being defined or is false, we want to use the default.
                 return $configpath;
             }
 
@@ -287,9 +290,27 @@ class cache_config_phpunittest extends cache_config_writer {
     }
 }
 
+/**
+ * This is a deprecated class. It has been renamed to cache_config_testing.
+ *
+ * This was deprecated in Moodle 2.9 but will be removed at the next major release
+ * as it is only used during testing and its highly unlikely anyone has used this.
+ *
+ * @deprecated since 2.9
+ * @copyright  2014 Sam Hemelryk
+ * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
+ */
+class cache_config_phpunittest extends cache_config_testing {
+    // We can't do anything here to warn the user.
+    // The cache can be utilised before sessions have even been started.
+}
+
 /**
  * Dummy object for testing cacheable object interface and interaction
  *
+ * Wake from cache needs specific testing at times to ensure that during multiple
+ * cache get() requests it's possible to verify that it's getting woken each time.
+ *
  * @copyright  2012 Sam Hemelryk
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
@@ -304,21 +325,27 @@ class cache_phpunit_dummy_object extends stdClass implements cacheable_object {
      * @var string
      */
     public $property2;
+    /**
+     * Test property time for verifying wake is run at each get() call.
+     * @var float
+     */
+    public $propertytime;
     /**
      * Constructor
      * @param string $property1
      * @param string $property2
      */
-    public function __construct($property1, $property2) {
+    public function __construct($property1, $property2, $propertytime = null) {
         $this->property1 = $property1;
         $this->property2 = $property2;
+        $this->propertytime = $propertytime === null ? microtime(true) : $propertytime;
     }
     /**
      * Prepares this object for caching
      * @return array
      */
     public function prepare_to_cache() {
-        return array($this->property1.'_ptc', $this->property2.'_ptc');
+        return array($this->property1.'_ptc', $this->property2.'_ptc', $this->propertytime);
     }
     /**
      * Returns this object from the cache
@@ -326,7 +353,15 @@ class cache_phpunit_dummy_object extends stdClass implements cacheable_object {
      * @return cache_phpunit_dummy_object
      */
     public static function wake_from_cache($data) {
-        return new cache_phpunit_dummy_object(array_shift($data).'_wfc', array_shift($data).'_wfc');
+        $time = null;
+        if (!is_null($data[2])) {
+            // Windows 32bit microtime() resolution is 15ms, we ensure the time has moved forward.
+            do {
+                $time = microtime(true);
+            } while ($time == $data[2]);
+
+        }
+        return new cache_phpunit_dummy_object(array_shift($data).'_wfc', array_shift($data).'_wfc', $time);
     }
 }
 
@@ -408,6 +443,17 @@ class cache_phpunit_application extends cache_application {
         $key = $this->parse_key($key);
         return $this->static_acceleration_get($key);
     }
+
+    /**
+     * Purges only the static acceleration while leaving the rest of the store in tack.
+     *
+     * Used for behaving like you have loaded 2 pages, and reset static while the backing store
+     * still contains all the same data.
+     *
+     */
+    public function phpunit_static_acceleration_purge() {
+        $this->static_acceleration_purge();
+    }
 }
 
 /**
