@@ -35,9 +35,9 @@
 abstract class testing_util {
 
     /**
-     * @var int last value of db writes counter, used for db resetting
+     * @var string dataroot (likely to be $CFG->dataroot).
      */
-    public static $lastdbwrites = null;
+    private static $dataroot = null;
 
     /**
      * @var testing_data_generator
@@ -59,11 +59,68 @@ abstract class testing_util {
      */
     protected static $tablestructure = null;
 
+    /**
+     * @var array keep list of sequenceid used in a table.
+     */
+    private static $tablesequences = array();
+
     /**
      * @var array original structure of all database tables
      */
     protected static $sequencenames = null;
 
+    /**
+     * @var string name of the json file where we store the list of dataroot files to not reset during reset_dataroot.
+     */
+    private static $originaldatafilesjson = 'originaldatafiles.json';
+
+    /**
+     * @var boolean set to true once $originaldatafilesjson file is created.
+     */
+    private static $originaldatafilesjsonadded = false;
+
+    /**
+     * @var int next sequence value for a single test cycle.
+     */
+    protected static $sequencenextstartingid = null;
+
+    /**
+     * Return the name of the JSON file containing the init filenames.
+     *
+     * @static
+     * @return string
+     */
+    public static function get_originaldatafilesjson() {
+        return self::$originaldatafilesjson;
+    }
+
+    /**
+     * Return the dataroot. It's useful when mocking the dataroot when unit testing this class itself.
+     *
+     * @static
+     * @return string the dataroot.
+     */
+    public static function get_dataroot() {
+        global $CFG;
+
+        //  By default it's the test framework dataroot.
+        if (empty(self::$dataroot)) {
+            self::$dataroot = $CFG->dataroot;
+        }
+
+        return self::$dataroot;
+    }
+
+    /**
+     * Set the dataroot. It's useful when mocking the dataroot when unit testing this class itself.
+     *
+     * @param string $dataroot the dataroot of the test framework.
+     * @static
+     */
+    public static function set_dataroot($dataroot) {
+        self::$dataroot = $dataroot;
+    }
+
     /**
      * Returns the testing framework name
      * @static
@@ -99,7 +156,7 @@ abstract class testing_util {
 
         $framework = self::get_framework();
 
-        if (!file_exists($CFG->dataroot . '/' . $framework . 'testdir.txt')) {
+        if (!file_exists(self::get_dataroot() . '/' . $framework . 'testdir.txt')) {
             // this is already tested in bootstrap script,
             // but anyway presence of this file means the dataroot is for testing
             return false;
@@ -128,7 +185,7 @@ abstract class testing_util {
 
         $framework = self::get_framework();
 
-        $datarootpath = $CFG->dataroot . '/' . $framework;
+        $datarootpath = self::get_dataroot() . '/' . $framework;
         if (!file_exists($datarootpath . '/tabledata.ser') or !file_exists($datarootpath . '/tablestructure.ser')) {
             return false;
         }
@@ -178,12 +235,12 @@ abstract class testing_util {
             }
         }
         $data = serialize($data);
-        $datafile = $CFG->dataroot . '/' . $framework . '/tabledata.ser';
+        $datafile = self::get_dataroot() . '/' . $framework . '/tabledata.ser';
         file_put_contents($datafile, $data);
         testing_fix_file_permissions($datafile);
 
         $structure = serialize($structure);
-        $structurefile = $CFG->dataroot . '/' . $framework . '/tablestructure.ser';
+        $structurefile = self::get_dataroot() . '/' . $framework . '/tablestructure.ser';
         file_put_contents($structurefile, $structure);
         testing_fix_file_permissions($structurefile);
     }
@@ -201,7 +258,7 @@ abstract class testing_util {
         set_config($framework . 'test', $hash);
 
         // hash all plugin versions - helps with very fast detection of db structure changes
-        $hashfile = $CFG->dataroot . '/' . $framework . '/versionshash.txt';
+        $hashfile = self::get_dataroot() . '/' . $framework . '/versionshash.txt';
         file_put_contents($hashfile, $hash);
         testing_fix_file_permissions($hashfile);
     }
@@ -212,17 +269,15 @@ abstract class testing_util {
      * @return array  $table=>$records
      */
     protected static function get_tabledata() {
-        global $CFG;
-
-        $framework = self::get_framework();
-
-        $datafile = $CFG->dataroot . '/' . $framework . '/tabledata.ser';
-        if (!file_exists($datafile)) {
-            // Not initialised yet.
-            return array();
-        }
-
         if (!isset(self::$tabledata)) {
+            $framework = self::get_framework();
+
+            $datafile = self::get_dataroot() . '/' . $framework . '/tabledata.ser';
+            if (!file_exists($datafile)) {
+                // Not initialised yet.
+                return array();
+            }
+
             $data = file_get_contents($datafile);
             self::$tabledata = unserialize($data);
         }
@@ -240,17 +295,15 @@ abstract class testing_util {
      * @return array $table=>$records
      */
     public static function get_tablestructure() {
-        global $CFG;
-
-        $framework = self::get_framework();
-
-        $structurefile = $CFG->dataroot . '/' . $framework . '/tablestructure.ser';
-        if (!file_exists($structurefile)) {
-            // Not initialised yet.
-            return array();
-        }
-
         if (!isset(self::$tablestructure)) {
+            $framework = self::get_framework();
+
+            $structurefile = self::get_dataroot() . '/' . $framework . '/tablestructure.ser';
+            if (!file_exists($structurefile)) {
+                // Not initialised yet.
+                return array();
+            }
+
             $data = file_get_contents($structurefile);
             self::$tablestructure = unserialize($data);
         }
@@ -310,9 +363,10 @@ abstract class testing_util {
                     // incorrect table match caused by _
                     continue;
                 }
+
                 if (!is_null($info->auto_increment)) {
                     $table = preg_replace('/^'.preg_quote($prefix, '/').'/', '', $table);
-                    if ($info->auto_increment == 1) {
+                    if (isset(self::$tablesequences[$table]) && ($info->auto_increment == self::$tablesequences[$table])) {
                         $empties[$table] = $table;
                     }
                 }
@@ -362,6 +416,35 @@ abstract class testing_util {
         }
     }
 
+    /**
+     * Determine the next unique starting id sequences.
+     *
+     * @static
+     * @param array $records The records to use to determine the starting value for the table.
+     * @param string $table table name.
+     * @return int The value the sequence should be set to.
+     */
+    private static function get_next_sequence_starting_value($records, $table) {
+        if (isset(self::$tablesequences[$table])) {
+            return self::$tablesequences[$table];
+        }
+
+        $id = self::$sequencenextstartingid;
+
+        // If there are records, calculate the minimum id we can use.
+        // It must be bigger than the last record's id.
+        if (!empty($records)) {
+            $lastrecord = end($records);
+            $id = max($id, $lastrecord->id + 1);
+        }
+
+        self::$sequencenextstartingid = $id + 1000;
+
+        self::$tablesequences[$table] = $id;
+
+        return $id;
+    }
+
     /**
      * Reset all database sequences to initial values.
      *
@@ -381,18 +464,24 @@ abstract class testing_util {
             return;
         }
 
+        // If all starting Id's are the same, it's difficult to detect coding and testing
+        // errors that use the incorrect id in tests.  The classic case is cmid vs instance id.
+        // To reduce the chance of the coding error, we start sequences at different values where possible.
+        // In a attempt to avoid tables with existing id's we start at a high number.
+        // Reset the value each time all database sequences are reset.
+        if (defined('PHPUNIT_SEQUENCE_START') and PHPUNIT_SEQUENCE_START) {
+            self::$sequencenextstartingid = PHPUNIT_SEQUENCE_START;
+        } else {
+            self::$sequencenextstartingid = 100000;
+        }
+
         $dbfamily = $DB->get_dbfamily();
         if ($dbfamily === 'postgres') {
             $queries = array();
             $prefix = $DB->get_prefix();
             foreach ($data as $table => $records) {
                 if (isset($structure[$table]['id']) and $structure[$table]['id']->auto_increment) {
-                    if (empty($records)) {
-                        $nextid = 1;
-                    } else {
-                        $lastrecord = end($records);
-                        $nextid = $lastrecord->id + 1;
-                    }
+                    $nextid = self::get_next_sequence_starting_value($records, $table);
                     $queries[] = "ALTER SEQUENCE {$prefix}{$table}_id_seq RESTART WITH $nextid";
                 }
             }
@@ -420,16 +509,10 @@ abstract class testing_util {
             foreach ($data as $table => $records) {
                 if (isset($structure[$table]['id']) and $structure[$table]['id']->auto_increment) {
                     if (isset($sequences[$table])) {
-                        if (empty($records)) {
-                            $nextid = 1;
-                        } else {
-                            $lastrecord = end($records);
-                            $nextid = $lastrecord->id + 1;
-                        }
+                        $nextid = self::get_next_sequence_starting_value($records, $table);
                         if ($sequences[$table] != $nextid) {
                             $DB->change_database_structure("ALTER TABLE {$prefix}{$table} AUTO_INCREMENT = $nextid");
                         }
-
                     } else {
                         // some problem exists, fallback to standard code
                         $DB->get_manager()->reset_sequence($table);
@@ -475,6 +558,7 @@ abstract class testing_util {
 
         } else {
             // note: does mssql support any kind of faster reset?
+            // This also implies mssql will not use unique sequence values.
             if (is_null($empties)) {
                 $empties = self::guess_unmodified_empty_tables();
             }
@@ -497,10 +581,6 @@ abstract class testing_util {
     public static function reset_database() {
         global $DB;
 
-        if (!is_null(self::$lastdbwrites) and self::$lastdbwrites == $DB->perf_get_writes()) {
-            return false;
-        }
-
         $tables = $DB->get_tables(false);
         if (!$tables or empty($tables['config'])) {
             // not installed yet
@@ -518,11 +598,74 @@ abstract class testing_util {
 
         $empties = self::guess_unmodified_empty_tables();
 
+        $borkedmysql = false;
+        if ($DB->get_dbfamily() === 'mysql') {
+            $version = $DB->get_server_info();
+            if (version_compare($version['version'], '5.6.0') == 1 and version_compare($version['version'], '5.6.16') == -1) {
+                // Everything that comes from Oracle is evil!
+                //
+                // See http://dev.mysql.com/doc/refman/5.6/en/alter-table.html
+                // You cannot reset the counter to a value less than or equal to to the value that is currently in use.
+                //
+                // From 5.6.16 release notes:
+                //   InnoDB: The ALTER TABLE INPLACE algorithm would fail to decrease the auto-increment value.
+                //           (Bug #17250787, Bug #69882)
+                $borkedmysql = true;
+
+            } else if (version_compare($version['version'], '10.0.0') == 1) {
+                // And MariaDB is no better!
+                // Let's hope they pick the patch sometime later...
+                $borkedmysql = true;
+            }
+        }
+
+        if ($borkedmysql) {
+            $mysqlsequences = array();
+            $prefix = $DB->get_prefix();
+            $rs = $DB->get_recordset_sql("SHOW TABLE STATUS LIKE ?", array($prefix.'%'));
+            foreach ($rs as $info) {
+                $table = strtolower($info->name);
+                if (strpos($table, $prefix) !== 0) {
+                    // Incorrect table match caused by _ char.
+                    continue;
+                }
+                if (!is_null($info->auto_increment)) {
+                    $table = preg_replace('/^'.preg_quote($prefix, '/').'/', '', $table);
+                    $mysqlsequences[$table] = $info->auto_increment;
+                }
+            }
+        }
+
         foreach ($data as $table => $records) {
+            if ($borkedmysql) {
+                if (empty($records)) {
+                    if (!isset($empties[$table])) {
+                        // Table has been modified and is not empty.
+                        $DB->delete_records($table, null);
+                    }
+                    continue;
+                }
+
+                if (isset($structure[$table]['id']) and $structure[$table]['id']->auto_increment) {
+                    $current = $DB->get_records($table, array(), 'id ASC');
+                    if ($current == $records) {
+                        if (isset($mysqlsequences[$table]) and $mysqlsequences[$table] == $structure[$table]['id']->auto_increment) {
+                            continue;
+                        }
+                    }
+                }
+
+                // Use TRUNCATE as a workaround and reinsert everything.
+                $DB->delete_records($table, null);
+                foreach ($records as $record) {
+                    $DB->import_record($table, $record, false, true);
+                }
+                continue;
+            }
+
             if (empty($records)) {
-                if (isset($empties[$table])) {
-                    // table was not modified and is empty
-                } else {
+                if (!isset($empties[$table])) {
+                    // Table has been modified and is not empty.
                     $DB->delete_records($table, array());
                 }
                 continue;
@@ -569,8 +712,6 @@ abstract class testing_util {
             }
         }
 
-        self::$lastdbwrites = $DB->perf_get_writes();
-
         return true;
     }
 
@@ -584,18 +725,42 @@ abstract class testing_util {
 
         $childclassname = self::get_framework() . '_util';
 
-        $handle = opendir($CFG->dataroot);
+        // Do not delete automatically installed files.
+        self::skip_original_data_files($childclassname);
+
+        // Clear file status cache, before checking file_exists.
+        clearstatcache();
+
+        // Clean up the dataroot folder.
+        $handle = opendir(self::get_dataroot());
         while (false !== ($item = readdir($handle))) {
             if (in_array($item, $childclassname::$datarootskiponreset)) {
                 continue;
             }
-            if (is_dir("$CFG->dataroot/$item")) {
-                remove_dir("$CFG->dataroot/$item", false);
+            if (is_dir(self::get_dataroot()."/$item")) {
+                remove_dir(self::get_dataroot()."/$item", false);
             } else {
-                unlink("$CFG->dataroot/$item");
+                unlink(self::get_dataroot()."/$item");
             }
         }
         closedir($handle);
+
+        // Clean up the dataroot/filedir folder.
+        if (file_exists(self::get_dataroot() . '/filedir')) {
+            $handle = opendir(self::get_dataroot() . '/filedir');
+            while (false !== ($item = readdir($handle))) {
+                if (in_array('filedir' . DIRECTORY_SEPARATOR . $item, $childclassname::$datarootskiponreset)) {
+                    continue;
+                }
+                if (is_dir(self::get_dataroot()."/filedir/$item")) {
+                    remove_dir(self::get_dataroot()."/filedir/$item", false);
+                } else {
+                    unlink(self::get_dataroot()."/filedir/$item");
+                }
+            }
+            closedir($handle);
+        }
+
         make_temp_directory('');
         make_cache_directory('');
         make_localcache_directory('');
@@ -607,6 +772,88 @@ abstract class testing_util {
         cache_helper::purge_all();
     }
 
+    /**
+     * Gets a text-based site version description.
+     *
+     * @return string The site info
+     */
+    public static function get_site_info() {
+        global $CFG;
+
+        $output = '';
+
+        // All developers have to understand English, do not localise!
+        $env = self::get_environment();
+
+        $output .= "Moodle ".$env['moodleversion'];
+        if ($hash = self::get_git_hash()) {
+            $output .= ", $hash";
+        }
+        $output .= "\n";
+
+        // Add php version.
+        require_once($CFG->libdir.'/environmentlib.php');
+        $output .= "Php: ". normalize_version($env['phpversion']);
+
+        // Add database type and version.
+        $output .= ", " . $env['dbtype'] . ": " . $env['dbversion'];
+
+        // OS details.
+        $output .= ", OS: " . $env['os'] . "\n";
+
+        return $output;
+    }
+
+    /**
+     * Try to get current git hash of the Moodle in $CFG->dirroot.
+     * @return string null if unknown, sha1 hash if known
+     */
+    public static function get_git_hash() {
+        global $CFG;
+
+        // This is a bit naive, but it should mostly work for all platforms.
+
+        if (!file_exists("$CFG->dirroot/.git/HEAD")) {
+            return null;
+        }
+
+        $headcontent = file_get_contents("$CFG->dirroot/.git/HEAD");
+        if ($headcontent === false) {
+            return null;
+        }
+
+        $headcontent = trim($headcontent);
+
+        // If it is pointing to a hash we return it directly.
+        if (strlen($headcontent) === 40) {
+            return $headcontent;
+        }
+
+        if (strpos($headcontent, 'ref: ') !== 0) {
+            return null;
+        }
+
+        $ref = substr($headcontent, 5);
+
+        if (!file_exists("$CFG->dirroot/.git/$ref")) {
+            return null;
+        }
+
+        $hash = file_get_contents("$CFG->dirroot/.git/$ref");
+
+        if ($hash === false) {
+            return null;
+        }
+
+        $hash = trim($hash);
+
+        if (strlen($hash) != 40) {
+            return null;
+        }
+
+        return $hash;
+    }
+
     /**
      * Drop the whole test database
      * @static
@@ -656,17 +903,125 @@ abstract class testing_util {
         $framework = self::get_framework();
         $childclassname = $framework . '_util';
 
-        $files = scandir($CFG->dataroot . '/' . $framework);
+        $files = scandir(self::get_dataroot() . '/'  . $framework);
         foreach ($files as $file) {
             if (in_array($file, $childclassname::$datarootskipondrop)) {
                 continue;
             }
-            $path = $CFG->dataroot . '/' . $framework . '/' . $file;
+            $path = self::get_dataroot() . '/' . $framework . '/' . $file;
             if (is_dir($path)) {
                 remove_dir($path, false);
             } else {
                 unlink($path);
             }
         }
+
+        $jsonfilepath = self::get_dataroot() . '/' . self::$originaldatafilesjson;
+        if (file_exists($jsonfilepath)) {
+            // Delete the json file.
+            unlink($jsonfilepath);
+            // Delete the dataroot filedir.
+            remove_dir(self::get_dataroot() . '/filedir', false);
+        }
+    }
+
+    /**
+     * Skip the original dataroot files to not been reset.
+     *
+     * @static
+     * @param string $utilclassname the util class name..
+     */
+    protected static function skip_original_data_files($utilclassname) {
+        $jsonfilepath = self::get_dataroot() . '/' . self::$originaldatafilesjson;
+        if (file_exists($jsonfilepath)) {
+
+            $listfiles = file_get_contents($jsonfilepath);
+
+            // Mark each files as to not be reset.
+            if (!empty($listfiles) && !self::$originaldatafilesjsonadded) {
+                $originaldatarootfiles = json_decode($listfiles);
+                // Keep the json file. Only drop_dataroot() should delete it.
+                $originaldatarootfiles[] = self::$originaldatafilesjson;
+                $utilclassname::$datarootskiponreset = array_merge($utilclassname::$datarootskiponreset,
+                    $originaldatarootfiles);
+                self::$originaldatafilesjsonadded = true;
+            }
+        }
+    }
+
+    /**
+     * Save the list of the original dataroot files into a json file.
+     */
+    protected static function save_original_data_files() {
+        global $CFG;
+
+        $jsonfilepath = self::get_dataroot() . '/' . self::$originaldatafilesjson;
+
+        // Save the original dataroot files if not done (only executed the first time).
+        if (!file_exists($jsonfilepath)) {
+
+            $listfiles = array();
+            $currentdir = 'filedir' . DIRECTORY_SEPARATOR . '.';
+            $parentdir = 'filedir' . DIRECTORY_SEPARATOR . '..';
+            $listfiles[$currentdir] = $currentdir;
+            $listfiles[$parentdir] = $parentdir;
+
+            $filedir = self::get_dataroot() . '/filedir';
+            if (file_exists($filedir)) {
+                $directory = new RecursiveDirectoryIterator($filedir);
+                foreach (new RecursiveIteratorIterator($directory) as $file) {
+                    if ($file->isDir()) {
+                        $key = substr($file->getPath(), strlen(self::get_dataroot() . '/'));
+                    } else {
+                        $key = substr($file->getPathName(), strlen(self::get_dataroot() . '/'));
+                    }
+                    $listfiles[$key] = $key;
+                }
+            }
+
+            // Save the file list in a JSON file.
+            $fp = fopen($jsonfilepath, 'w');
+            fwrite($fp, json_encode(array_values($listfiles)));
+            fclose($fp);
+        }
+    }
+
+    /**
+     * Return list of environment versions on which tests will run.
+     * Environment includes:
+     * - moodleversion
+     * - phpversion
+     * - dbtype
+     * - dbversion
+     * - os
+     *
+     * @return array
+     */
+    public static function get_environment() {
+        global $CFG, $DB;
+
+        $env = array();
+
+        // Add moodle version.
+        $release = null;
+        require("$CFG->dirroot/version.php");
+        $env['moodleversion'] = $release;
+
+        // Add php version.
+        $phpversion = phpversion();
+        $env['phpversion'] = $phpversion;
+
+        // Add database type and version.
+        $dbtype = $CFG->dbtype;
+        $dbinfo = $DB->get_server_info();
+        $dbversion = $dbinfo['version'];
+        $env['dbtype'] = $dbtype;
+        $env['dbversion'] = $dbversion;
+
+        // OS details.
+        $osdetails = php_uname('s') . " " . php_uname('r') . " " . php_uname('m');
+        $env['os'] = $osdetails;
+
+        return $env;
     }
 }
