@@ -491,20 +491,6 @@ function lti_grade_item_delete($basiclti) {
     return grade_update('mod/lti', $basiclti->course, 'mod', 'lti', $basiclti->id, 0, null, array('deleted' => 1));
 }
 
-function lti_extend_settings_navigation($settings, $parentnode) {
-    global $PAGE;
-
-    if (has_capability('mod/lti:manage', context_module::instance($PAGE->cm->id))) {
-        $keys = $parentnode->get_children_key_list();
-
-        $node = navigation_node::create('Submissions',
-            new moodle_url('/mod/lti/grade.php', array('id' => $PAGE->cm->id)),
-            navigation_node::TYPE_SETTING, null, 'mod_lti_submissions');
-
-        $parentnode->add_node($node, $keys[1]);
-    }
-}
-
 /**
  * Log post actions
  *
@@ -522,3 +508,31 @@ function lti_get_post_actions() {
 function lti_get_view_actions() {
     return array('view all', 'view');
 }
+
+/**
+ * Mark the activity completed (if required) and trigger the course_module_viewed event.
+ *
+ * @param  stdClass $lti        lti object
+ * @param  stdClass $course     course object
+ * @param  stdClass $cm         course module object
+ * @param  stdClass $context    context object
+ * @since Moodle 3.0
+ */
+function lti_view($lti, $course, $cm, $context) {
+
+    // Trigger course_module_viewed event.
+    $params = array(
+        'context' => $context,
+        'objectid' => $lti->id
+    );
+
+    $event = \mod_lti\event\course_module_viewed::create($params);
+    $event->add_record_snapshot('course_modules', $cm);
+    $event->add_record_snapshot('course', $course);
+    $event->add_record_snapshot('lti', $lti);
+    $event->trigger();
+
+    // Completion.
+    $completion = new completion_info($course);
+    $completion->set_module_viewed($cm);
+}
